import os.path
import sys

import inforion.ionapi.model.inforlogin as inforlogin
import validators
from inforion.helper.urlsplit import spliturl
from inforion.ionapi.controller import *
from inforion.ionapi.model import *
from inforion.transformation.transform import parallelize_tranformation

# from inforion.ionapi.ionbasic import ionbasic

# from ionbasic import load_config

# Codee Junaid


def main_load(
        url=None,
        ionfile=None,
        program=None,
        method=None,
        dataframe=None,
        outputfile=None,
        start=None,
        end=None,
):

    if validators.url(url) != True:
        return "Error: URL is not valid"

    if os.path.exists(ionfile) == False:
        return "Error: File does not exist"
    else:
        inforlogin.load_config(ionfile)

    result = spliturl(url)

    if "Call" in result:
        if len(result["Call"]) > 0:
            if result["Call"] == "execute":
                inforlogin.load_config(ionfile)
                token = inforlogin.login()

                headers = inforlogin.header()
                if "Bearer" not in headers["Authorization"]:
                    return "Error: InforION Login is not working"
                if start is None or end is None:
                    return execute(url, headers, program, method, dataframe,
                                   outputfile)

                else:
                    return execute(url, headers, program, method, dataframe,
                                   outputfile, start, end)

            if result["Call"] == "executeSnd":

                config = inforlogin.load_config(ionfile)
                token = inforlogin.login()

                headers = inforlogin.header(token)
                if "Bearer" not in headers["Authorization"]:
                    return "InforION Login is not working"
                return executeSnd(url, headers, program, method, dataframe,
                                  outputfile, start, end)
            if result["Call"] == "executeAsyncSnd":

                config = inforlogin.load_config(ionfile)
                token = inforlogin.login()

                headers = inforlogin.header(token)
                if "Bearer" not in headers["Authorization"]:
                    return "InforION Login is not working"
                return executeAsyncSnd(url, headers, program, method,
                                       dataframe, outputfile, start, end)

    if method == "checklogin":
        token = inforlogin.login()
        headers = inforlogin.header()
        return headers["Authorization"]


def main_transformation(mappingfile=None,
                        mainsheet=None,
                        stagingdata=None,
                        outputfile=None):

    if mappingfile is None:
        return "Error: Mapping file path missing"

    if os.path.exists(mappingfile) == False:
        return "Error: Mapping file does not exist"

    if mainsheet is None:
        return "Error: Main sheet name is empty"

    if stagingdata.empty:
        return "Error: Data frame is empty"

    return parallelize_tranformation(mappingfile, mainsheet, stagingdata,
                                     outputfile)



import requests
from inforion.ionapi.model import inforlogin


def get_v1_payloads_list(filter=None, sort=None, page=None, records=None):
    """
    List data object properties using a filter.
    """
    url = inforlogin.base_url() + "/IONSERVICES/datalakeapi/v1/payloads/list"
    headers = inforlogin.header()
    payload = {}

    if filter is not None:
        payload["filter"] = filter

    if sort is not None:
        payload["sort"] = sort

    if page is not None:
        payload["page"] = page

    if records is not None:
        payload["records"] = records

    res = requests.get(url, headers=headers, params=payload)
    return res


def get_v1_payloads_stream_by_id(dl_id):
    """
    Retrieve payload based on id from datalake.
    """
    url = inforlogin.base_url() + "/IONSERVICES/datalakeapi/v1/payloads/streambyid"
    headers = inforlogin.header()
    payload = {"datalakeId": dl_id}
    res = requests.get(url, headers=headers, params=payload)
    return res


def delete_v1_purge_id(ids):
    """
    Deletes Data Objects based on the given Data Object identifiers.
    """
    url = inforlogin.base_url() + "/IONSERVICES/datalakeapi/v1/purge/ids"
    headers = inforlogin.header()
    payload = {"id": ids}
    res = requests.delete(url, headers=headers, params=payload)
    return res



from enum import Enum
import requests
import inforion.ionapi.model.inforlogin as inforlogin
import logging as log
import json


class ObjectSchemaType(Enum):
    ANY = 'ANY'
    BOD = 'BOD'
    DSV = 'DSV'
    JSON = 'JSON'


def get_datacatalog_ping():
    url = inforlogin.base_url() + '/IONSERVICES/datacatalog/v1/status/ping'
    headers = inforlogin.header()
    res = requests.get(url, headers=headers)
    log.info('datacatalog ping: {}'.format(res.content))
    return res


def delete_datacatalog_object(object_name):
    url = inforlogin.base_url() + '/IONSERVICES/datacatalog/v1/object/{}'.format(object_name)
    headers = inforlogin.header()
    res = requests.delete(url, headers=headers)
    log.info('datacatalog delete: {}'.format(res.content))
    return res


def post_datacatalog_object(object_name, object_type: ObjectSchemaType, schema, properties):

    if object_type == ObjectSchemaType.ANY and (schema is not None or properties is not None):
        raise ValueError('Schema and properties should be None')

    if (object_type == ObjectSchemaType.DSV or object_type == ObjectSchemaType.DSV) and schema is None:
        raise ValueError('Schema cannot be None')

    url = inforlogin.base_url() + '/IONSERVICES/datacatalog/v1/object'
    headers = inforlogin.header()
    data = {
        "name": object_name,
        "type": object_type.value,
        "schema": schema,
        "properties": properties
    }
    res = requests.post(url, headers=headers, data=json.dumps(data))
    log.info('datacatalog post: {}'.format(res.content))
    return res



import pandas as pd
import numpy as np
import requests
import inforion
import json

import time
import progressbar

# import grequests

from pandas import compat

import xlsxwriter

from requests_oauthlib import OAuth2Session
from requests.auth import HTTPBasicAuth
from oauthlib.oauth2 import BackendApplicationClient


from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

import inforion.ionapi.controller as controller
import inforion.helper.filehandling as filehandling

from openpyxl import load_workbook
from openpyxl import Workbook
from openpyxl.chart import BarChart, Series, Reference

# import sendresults, saveresults
# from inforion.ionapi.model import

DEFAULT_TIMEOUT = 50  # seconds
MaxChunk = 100


def execute(
    url, headers, program, methode, dataframe, outputfile=None, start=0, end=None
):

    df = dataframe

    df = df.replace(np.nan, "", regex=True)
    df = df.astype(str)

    data = {"program": program, "cono": 409}
    data1 = {}
    a = []

    chunk = MaxChunk
    if end is not None:
        df = df[start:end].copy(deep=False)
        df = df.reset_index(drop=True)
        # print (df.head(10))

    # else:
    total_rows = df.shape[0]
    total_rows = int(total_rows)

    methode = methode.split(",")
    methode_count = len(methode)

    print("Number of rows " + str(total_rows))

    with progressbar.ProgressBar(max_value=total_rows) as bar:
        for index, row in df.iterrows():

            bar.update(index)

            row = row.to_json()
            row = json.loads(row)

            for i in methode:
                data1["transaction"] = i
                data1["record"] = row
                a.append(data1.copy())

            if chunk == 0:
                data["transactions"] = a

                r = controller.sendresults(url, headers, data)
                df, data, chunk = controller.saveresults(
                    r, df, program, index, chunk, MaxChunk, methode_count
                )
                data1 = {}
                a = []

            else:
                chunk = chunk - 1

        data["transactions"] = a

        r = controller.sendresults(url, headers, data)
        index = index + 1
        df, data, chunk = controller.saveresults(
            r, df, methode, index, chunk, MaxChunk, methode_count
        )

    # df = df.replace(np.nan, '', regex=True)
    # df = df.astype(str)

    if outputfile is not None:
        print("Save to file: " + outputfile)
        filehandling.savetodisk(outputfile, df)

        df_results = getSuccessGraphDataframe(df)
        createGraph(outputfile, df_results)

    return df


def executeSnd(
    url, headers, program, methode, dataframe, outputfile=None, start=0, end=None
):

    df = dataframe

    data = {"program": program, "cono": 409}

    methode = methode.split(",")
    len(methode)
    data1 = {}

    chunk = MaxChunk
    if end is not None:
        df = df[start:end].copy(deep=False)
        df = df.reset_index(drop=True)

    # else:
    total_rows = df.shape[0]
    total_rows = int(total_rows)

    print("Number of rows " + str(total_rows))

    a = []
    with progressbar.ProgressBar(max_value=total_rows) as bar:
        for index, row in df.iterrows():

            bar.update(index)

            row = row.to_json()
            row = json.loads(row)

            for i in methode:
                data1["transaction"] = i
                data1["record"] = row
                a.append(data1.copy())

        data["transactions"] = a

        index = index + 1

    print(data)

    r = controller.sendresults(url, headers, data)

    df, data, chunk = controller.saveresults(r, df, methode, index, chunk)

    df = df.replace(np.nan, "", regex=True)
    df = df.astype(str)

    if outputfile is not None:
        print("Save to file: " + outputfile)
        filehandling.savetodisk(outputfile, df)

    return df


def executeAsyncSnd(
    url, headers, program, methode, dataframe, _outputfile=None, start=0, end=None
):

    print("Still in Beta")

    df = dataframe

    data = {"program": program, "cono": 409}

    methode = methode.split(",")
    len(methode)
    data1 = {}
    if end is not None:
        df = df[start:end].copy(deep=False)
        df = df.reset_index(drop=True)
        # print (df.head(10))

    # else:
    total_rows = df.shape[0]
    total_rows = int(total_rows)

    print("Number of rows " + str(total_rows))

    a = []
    with progressbar.ProgressBar(max_value=total_rows) as bar:
        for index, row in df.iterrows():

            bar.update(index)

            row = row.to_json()
            row = json.loads(row)

            for i in methode:
                data1["transaction"] = i
                data1["record"] = row
                a.append(data1.copy())

        data["transactions"] = a

        print(data)

        r = controller.sendresults(url, headers, data, stream=True)
        index = index + 1
        # df,data,chunk = saveresults(r,df,methode,index,chunk)

    print(r)

    """

    df = df.replace(np.nan, '', regex=True)
    df = df.astype(str)
    
    if outputfile is not None:
        print ('Save to file: ' + outputfile)
        filehandling.savetodisk(outputfile,df)
    
    return df

    """


def getSuccessGraphDataframe(df):
    results = {}
    for _, row in df.iterrows():
        msgs = row["MESSAGE"].split("|")
        for msg in msgs:
            if msg:
                key = msg[0 : msg.index(":")]
                val = msg[msg.index(":") + 1 :]

                if key not in results:
                    results[key] = {}
                    results[key]["success"] = 0
                    results[key]["fail"] = 0

                if "OK" in val or "ist bereits vorhanden" in val:
                    results[key]["success"] = results[key]["success"] + 1
                else:
                    results[key]["fail"] = results[key]["fail"] + 1

    success = []
    fail = []
    prog = []
    for key in results:
        prog.append(key)
        success.append(results[key]["success"])
        fail.append(results[key]["fail"])

    df = pd.DataFrame({"success": success, "fail": fail}, index=prog)
    return df


def createGraph(excel_file, df):
    sheet_name = "Data"
    sheet_name_graphs = "Graphs"

    writer = pd.ExcelWriter(excel_file, engine="openpyxl")
    book = load_workbook(excel_file)
    writer.book = book
    writer.sheets = dict((ws.title, ws) for ws in book.worksheets)
    ws_graph = writer.book.create_sheet(sheet_name_graphs)
    df.to_excel(writer, sheet_name=sheet_name)

    ws = writer.sheets[sheet_name]

    chart1 = BarChart()
    chart1.type = "col"
    chart1.style = 10
    chart1.title = "ETL Results"
    chart1.y_axis.title = "Count"
    chart1.x_axis.title = "Programm"

    data = Reference(ws, min_col=2, min_row=1, max_row=7, max_col=3)
    cats = Reference(ws, min_col=1, min_row=2, max_row=7)
    chart1.add_data(data, titles_from_data=True)
    chart1.set_categories(cats)
    chart1.shape = 4
    chart1.height = 12
    chart1.width = 20
    # chart1.x_axis = 10
    # chart1.y_axis = 10

    ws_graph.add_chart(chart1, "A10")
    writer.save()
